Option Compare Binary
Option Explicit On
Option Strict On

Imports System
Imports System.ComponentModel
Imports System.Globalization
Imports System.Runtime.InteropServices

''' <summary>Represents a Win32 resource.</summary>
Public Class ResourceInfo
    Private m_Type As ResourceType
    Private m_Id As ResourceId
    Private m_Language As Short
    Private m_Data() As Byte
    Private m_DataAssigned As Boolean
    Private m_ModuleHandle As IntPtr

    ''' <summary>Initializes a new instance of the class.</summary>
    Public Sub New()
        Me.Language = NativeMethods.NeutralLanguage
    End Sub

    ''' <summary>Initializes a new instance of the class.</summary>
    ''' <param name="Type">The type of the resource.</param>
    ''' <param name="Id">The identifier of the resource.</param>
    Public Sub New( _
        ByVal Type As ResourceType, _
        ByVal Id As ResourceId _
    )
        Me.New()
        Me.Type = Type
        Me.Id = Id
    End Sub

    ''' <summary>Initializes a new instance of the class.</summary>
    ''' <param name="Type">The type of the resource.</param>
    ''' <param name="Id">The identifier of the resource.</param>
    ''' <param name="Language">The language of the resource.</param>
    Public Sub New( _
        ByVal Type As ResourceType, _
        ByVal Id As ResourceId, _
        ByVal Language As Short _
    )
        Me.New(Type, Id)
        Me.Language = Language
    End Sub

    ''' <summary>Initializes a new instance of the class.</summary>
    ''' <param name="Type">The type of the resource.</param>
    ''' <param name="Id">The identifier of the resource.</param>
    ''' <param name="ModuleHandle">
    '''   The module handle associated with the resource.
    ''' </param>
    Friend Sub New( _
        ByVal Type As ResourceType, _
        ByVal Id As ResourceId, _
        ByVal ModuleHandle As IntPtr _
    )
        Me.New(Type, Id)
        Me.ModuleHandle = ModuleHandle
    End Sub

    ''' <summary>Initializes a new instance of the class.</summary>
    ''' <param name="Type">The type of the resource.</param>
    ''' <param name="Id">The identifier of the resource.</param>
    ''' <param name="Language">The language of the resource.</param>
    ''' <param name="ModuleHandle">
    '''   The module handle associated with the resource.
    ''' </param>
    Friend Sub New( _
        ByVal Type As ResourceType, _
        ByVal Id As ResourceId, _
        ByVal Language As Short, _
        ByVal ModuleHandle As IntPtr _
    )
        Me.New(Type, Id, Language)
        Me.ModuleHandle = ModuleHandle
    End Sub

    ''' <summary>
    '''   Gets or sets the module handle associated with the resource.
    ''' </summary>
    ''' <value>
    '''   The module handle; <c>IntPtr.Zero</c> means that no handle is
    '''   assigned.
    ''' </value>
    ''' <returns>
    '''   A <c>System.IntPtr</c> containing the module handle.
    ''' </returns>
    Public Property ModuleHandle() As IntPtr
        Get
            Return m_ModuleHandle
        End Get
        Set(ByVal Value As IntPtr)
            m_ModuleHandle = Value
        End Set
    End Property

    ''' <summary>Gets or sets the identifier of the resource.</summary>
    ''' <value>The resource identifier of the resource.</value>
    ''' <returns>A resource identifier identifying the resource.</returns>
    Public Property Id() As ResourceId
        Get
            Return m_Id
        End Get
        Set(ByVal Value As ResourceId)
            m_Id = Value
        End Set
    End Property

    ''' <summary>Sets the data of the resource.</summary>
    ''' <param name="Value">
    '''   A byte array containing the data of the resource.
    ''' </param>
    Public Sub SetData(ByVal Value() As Byte)
        m_Data = Value
        m_DataAssigned = True
    End Sub

    ''' <summary>Gets the data of the resource.</summary>
    ''' <returns>A byte array containing the data of the resource.</returns>
    Public Function GetData() As Byte()
        If Not m_DataAssigned Then

            ' Retrieve data.  The resource handle is simply the location in
            ' the memory and thus does not need to be freed.
            Dim hResource As IntPtr = _
                NativeMethods.FindResourceEx( _
                    Me.ModuleHandle, _
                    Me.Type, _
                    Me.Id, _
                    Me.Language _
                )
            If hResource = IntPtr.Zero Then
                Throw New Win32Exception()
            End If
            Me.SetData(GetResourceData(m_ModuleHandle, hResource))
        End If
        Return m_Data
    End Function

    ''' <summary>Gets or sets the type of the resource.</summary>
    ''' <value>The type of the resource.</value>
    ''' <returns>
    '''   A resource type representing the resource type of the resource.
    ''' </returns>
    Public Property Type() As ResourceType
        Get
            Return m_Type
        End Get
        Set(ByVal Value As ResourceType)
            m_Type = Value
        End Set
    End Property

    ''' <summary>
    '''   Gets or sets the language identifier of the resource.
    ''' </summary>
    ''' <value>The language of the resource.</value>
    ''' <returns>
    '''   A <c>System.Int16</c> object containing the language in
    '''   <c>LANGID</c> format.
    ''' </returns>
    Public Property Language() As Short
        Get
            Return m_Language
        End Get
        Set(ByVal Value As Short)
            m_Language = Value
        End Set
    End Property

    ''' <summary>Returns a string representing the object.</summary>
    ''' <returns>A string representing the object.</returns>
    Public Overrides Function ToString() As String
        Return _
            String.Format( _
                CultureInfo.InvariantCulture, _
                "Type={{{0}}}, Id={{{1}}}, Language={{{2}}}, ModuleHandle={{{3}}}", _
                Me.Type.ToString(), _
                Me.Id.ToString(), _
                NativeMethods.LangIdToTwoLetterCode(Me.Language), _
                Me.ModuleHandle.ToString() _
            )
    End Function

    Private Shared Function GetResourceData( _
        ByVal hModule As IntPtr, _
        ByVal hResource As IntPtr _
    ) As Byte()
        Dim hMemGlobal As IntPtr = _
            NativeMethods.LoadResource(hModule, hResource)
        If hMemGlobal = IntPtr.Zero Then
            Throw New Win32Exception()
        End If
        Try

            ' Unlocking is performed automatically.
            Dim MemPtr As IntPtr = NativeMethods.LockResource(hMemGlobal)
            If MemPtr = IntPtr.Zero Then
                Throw _
                    New Win32Exception( _
                        String.Format( _
                            CultureInfo.InvariantCulture, _
                            My.Resources.ExceptionResourceWithHandleXCouldNotBeLocked, _
                            hMemGlobal.ToString() _
                        ) _
                    )
            Else
                Dim ResSize As Integer = _
                    NativeMethods.SizeofResource(hModule, hResource)
                If ResSize > 0 Then
                    Dim Data(ResSize - 1) As Byte
                    Marshal.Copy(MemPtr, Data, 0, ResSize)
                    Return Data
                End If
            End If
        Catch
            Throw
        Finally
            If Not NativeMethods.FreeResource(hMemGlobal) Then
                'Throw _
                '    New Win32Exception( _
                '        "The resource with handle " & _
                '        hMemGlobal.ToString() & _
                '        " could not be freed because it was not found." _
                '    )
            End If
        End Try
        Return Nothing
    End Function
End Class