Option Compare Binary
Option Explicit On
Option Strict On

Imports MvpsOrg.Dotnet.Win32.ResLib.NativeMethods
Imports System
Imports System.Runtime.InteropServices

''' <summary>
'''   Represents a Win32 resource type.
''' </summary>
''' <completionlist cref="ResourceType"/>
Public MustInherit Class ResourceType

    ''' <summary>
    '''   Initializes a new instance of the class.
    ''' </summary>
    Friend Sub New()
        '
    End Sub

    ''' <summary>
    '''   Creates a resource type of appropriate type based on the
    '''   specified string pointer.
    ''' </summary>
    ''' <param name="Value">
    '''   The pointer from which the resource type should be created.
    ''' </param>
    ''' <returns>
    '''   A resource type object of appropriate type whose value is equal
    '''   to the specified identifier.
    ''' </returns>
    ''' <remarks>
    '''   <paramref name="Value"/> must be either a valid integer resource
    '''   type or a valid string pointer pointing to a resource type name.
    ''' </remarks>
    Public Shared Function FromLPStr(ByVal Value As IntPtr) As ResourceType
        If NativeMethods.IS_INTRESOURCE(Value) Then
            Return New IntegerResourceType(Value)
        Else
            Return New StringResourceType(Marshal.PtrToStringAuto(Value))
        End If
    End Function

    ''' <summary>
    '''   Validates the specified resource identifier.
    ''' </summary>
    ''' <param name="Value">
    '''   The integer resource identifier to be validated.
    ''' </param>
    ''' <returns>
    '''   <c>True</c> if the specified value is a valid integer resource
    '''   identifier; otherwise, <c>False</c>.
    ''' </returns>
    Public Shared Function IsIntegerResource(ByVal Value As IntPtr) As Boolean
        Return NativeMethods.IS_INTRESOURCE(Value)
    End Function

    ''' <summary>
    '''   Validates the specified resource identifier.
    ''' </summary>
    ''' <param name="Value">
    '''   The integer resource identifier to be validated.
    ''' </param>
    ''' <remarks>
    '''   <c>True</c> if the specified value is a valid integer resource
    '''   identifier; otherwise, <c>False</c>.
    ''' </remarks>
    Public Shared Function IsIntegerResource(ByVal Value As Integer) As Boolean
        Return NativeMethods.IS_INTRESOURCE(New IntPtr(Value))
    End Function

    ''' <summary>
    '''   Determines whether the two specified objects are equal.
    ''' </summary>
    ''' <param name="lhs">
    '''   The value on the left hand side of the comparison.
    ''' </param>
    ''' <param name="rhs">
    '''   The value on the right hand side of the comparison.
    ''' </param>
    ''' <returns>
    '''   <c>True</c> if the two specified objects are equal; otherwise,
    '''   <c>False</c>.
    ''' </returns>
    Public Shared Operator =( _
        ByVal lhs As ResourceType, _
        ByVal rhs As ResourceType _
    ) As Boolean
        If lhs Is rhs Then
            Return True
        ElseIf lhs Is Nothing OrElse rhs Is Nothing Then
            Return False
        ElseIf lhs.GetType() IsNot rhs.GetType() Then
            Return False
        ElseIf TypeOf lhs Is IntegerResourceType Then
            Return _
                DirectCast(lhs, IntegerResourceType).Value = _
                DirectCast(rhs, IntegerResourceType).Value
        ElseIf TypeOf lhs Is StringResourceType Then
            Return _
                DirectCast(lhs, StringResourceType).Value = _
                DirectCast(rhs, StringResourceType).Value
        Else
            Throw New NotSupportedException(My.Resources.ExceptionOperationNotSupportedForTypesOfArguments)
        End If
    End Operator

    ''' <summary>
    '''   Determines whether the two specified objects are different.
    ''' </summary>
    ''' <param name="lhs">
    '''   The value on the left hand side of the comparison.
    ''' </param>
    ''' <param name="rhs">
    '''   The value on the right hand side of the comparison.
    ''' </param>
    ''' <returns>
    '''   <c>True</c> if the two specified objects are different;
    '''   otherwise, <c>False</c>.
    ''' </returns>
    Public Shared Operator <>( _
        ByVal lhs As ResourceType, _
        ByVal rhs As ResourceType _
    ) As Boolean
        Return Not lhs = rhs
    End Operator

    ''' <summary>
    '''   Determines whether the specified object is equal to the current
    '''   object.
    ''' </summary>
    ''' <param name="obj">
    '''   The object to compare with the current object.
    ''' </param>
    ''' <returns>
    '''   <c>True</c> if the specified object is equal to the current
    '''   object; otherwise, <c>False</c>.
    ''' </returns>
    Public Overrides Function Equals(ByVal obj As Object) As Boolean
        If obj Is Nothing Then
            Return False
        ElseIf obj.GetType() IsNot Me.GetType() Then
            Return False
        ElseIf TypeOf obj Is IntegerResourceType Then
            Return _
                DirectCast(obj, IntegerResourceType).Value = _
                DirectCast(Me, IntegerResourceType).Value
        ElseIf TypeOf obj Is StringResourceType Then
            Return _
                DirectCast(obj, StringResourceType).Value = _
                DirectCast(Me, StringResourceType).Value
        Else
            Throw New NotSupportedException()
        End If
    End Function

    ''' <summary>
    '''   Serves as a hash function for this type. The return value is
    '''   suitable for use in hashing algorithms and data structures like a
    '''   hash table.
    ''' </summary>
    ''' <returns>A hash code for the current object.</returns>
    Public Overrides Function GetHashCode() As Integer
        Return MyBase.GetHashCode()
    End Function

    ''' <summary>Hardware-dependent cursor resource type.</summary>
    Public Shared ReadOnly Cursor As New IntegerResourceType(RT_Constants.RT_CURSOR)

    ''' <summary>Bitmap resource type.</summary>
    Public Shared ReadOnly Bitmap As New IntegerResourceType(RT_Constants.RT_BITMAP)

    ''' <summary>Hardware-dependent icon resource type.</summary>
    Public Shared ReadOnly Icon As New IntegerResourceType(RT_Constants.RT_ICON)

    ''' <summary>Menu resource type.</summary>
    Public Shared ReadOnly Menu As New IntegerResourceType(RT_Constants.RT_MENU)

    ''' <summary>Dialog resource type.</summary>
    Public Shared ReadOnly Dialog As New IntegerResourceType(RT_Constants.RT_DIALOG)

    ''' <summary>String resource type.</summary>
    Public Shared ReadOnly [String] As New IntegerResourceType(RT_Constants.RT_STRING)

    ''' <summary>Font directory resource type.</summary>
    Public Shared ReadOnly FontDirectory As New IntegerResourceType(RT_Constants.RT_FONTDIR)

    ''' <summary>Font resource type.</summary>
    Public Shared ReadOnly Font As New IntegerResourceType(RT_Constants.RT_FONT)

    ''' <summary>Accelerator table resource type.</summary>
    Public Shared ReadOnly Accelerator As New IntegerResourceType(RT_Constants.RT_ACCELERATOR)

    ''' <summary>
    '''   Application-defined resource (raw data) resource type.
    ''' </summary>
    Public Shared ReadOnly RCData As New IntegerResourceType(RT_Constants.RT_RCDATA)

    ''' <summary>Message-table entry resource type.</summary>
    Public Shared ReadOnly MessageTable As New IntegerResourceType(RT_Constants.RT_MESSAGETABLE)

    ''' <summary>Hardware-independent cursor resource type.</summary>
    Public Shared ReadOnly GroupCursor As New IntegerResourceType(RT_Constants.RT_GROUP_CURSOR)

    ''' <summary>Hardware-independent icon resource.</summary>
    Public Shared ReadOnly GroupIcon As New IntegerResourceType(RT_Constants.RT_GROUP_ICON)

    ''' <summary>Version resource type.</summary>
    Public Shared ReadOnly Version As New IntegerResourceType(RT_Constants.RT_VERSION)

    ''' <summary>Dialog include resource type.</summary>
    Public Shared ReadOnly DialogInclude As New IntegerResourceType(RT_Constants.RT_DLGINCLUDE)

    ''' <summary>Plug and Play resource type.</summary>
    Public Shared ReadOnly PlugPlay As New IntegerResourceType(RT_Constants.RT_PLUGPLAY)

    ''' <summary>VXD resource type.</summary>
    Public Shared ReadOnly Vxd As New IntegerResourceType(RT_Constants.RT_VXD)

    ''' <summary>Animated cursor resource type.</summary>
    Public Shared ReadOnly AnimatedCursor As New IntegerResourceType(RT_Constants.RT_ANICURSOR)

    ''' <summary>Animated icon resource type.</summary>
    Public Shared ReadOnly AnimatedIcon As New IntegerResourceType(RT_Constants.RT_ANIICON)

    ''' <summary>HTML resource type.</summary>
    Public Shared ReadOnly Html As New IntegerResourceType(RT_Constants.RT_HTML)

    ''' <summary>
    '''   Microsoft Windows XP Side-by-Side assembly XML manifest resource
    '''   type.
    ''' </summary>
    Public Shared ReadOnly Manifest As New IntegerResourceType(RT_Constants.RT_MANIFEST)
End Class