Option Compare Binary
Option Explicit On
Option Strict On

Imports Microsoft.VisualBasic
Imports MvpsOrg.Dotnet.Win32.ResLib
Imports System
Imports CollObjModel = System.Collections.ObjectModel
Imports System.IO
Imports System.Text

''' <summary>
'''   Provides the entry point of the application.
''' </summary>
Public Module Program

    ''' <summary>
    '''   The entry point of the application.
    ''' </summary>
    ''' <param name="Args">The array of command line arguments.</param>
    Public Sub Main(ByVal Args() As String)
        WriteLogo()

        ' Too many or too few parameters specified.
        If Args.Length = 0 OrElse Args.Length > 3 Then
            WriteUsage()
            Return
        End If
        If Args.Length = 1 Then
            Try
                Using Manager As New ResourceManager(Args(0))
                    WriteManifestResourceList(Manager)
                End Using
            Catch ex As Exception
                WriteError(ex.ToString())
            End Try
            Return
        End If
        Try
            Using Updater As New ResourceUpdater(Args(0))
                Updater.BeginUpdate()
                Dim Resource As New ResourceInfo()
                Resource.Type = ResourceType.Manifest

                ' Integer resource identifier specified.
                If Args.Length = 3 Then
                    Dim Id As Integer
                    If Integer.TryParse(Args(2), Id) Then
                        If ResourceType.IsIntegerResource(Id) Then
                            Resource.Id = New IntegerResourceId(Id)
                        Else
                            WriteError("'NumericResourceId' is not a valid resource identifier.")
                            Return
                        End If
                    Else
                        Resource.Id = New StringResourceId(Args(2))
                    End If
                Else
                    Resource.Id = ResourceId.CreateProcessManifest
                End If
                Dim Data() As Byte = My.Computer.FileSystem.ReadAllBytes(Args(1))
                Dim Length As Integer = Data.Length

                ' The length of the file in bytes must be dividable by
                ' four.  If this is not the case the array will be resized.
                Dim [Mod] As Integer = Length Mod 4
                If [Mod] > 0 Then
                    Length += 4 - [Mod]
                End If
                ReDim Preserve Data(Length - 1)
                Resource.SetData(Data)
                Updater.UpdateResource(Resource)
                Updater.EndUpdate()
                Console.WriteLine("Resources updated successfully.")
            End Using
        Catch ex As Exception
            WriteError(ex.ToString())
        End Try
    End Sub

    ''' <summary>
    '''   Writes an overview over the manifest resources contained in the
    '''   image file onto the console.
    ''' </summary>
    ''' <param name="Manager">
    '''   The resource manager providing access to resource information.
    ''' </param>
    Private Sub WriteManifestResourceList(ByVal Manager As ResourceManager)
        Dim Resources As CollObjModel.Collection(Of ResourceInfo) = _
            Manager.FindResources( _
                ResourceManager.FindResourceParameters.Type, _
                Nothing, _
                ResourceType.Manifest, _
                Nothing _
            )
        If Resources.Count = 0 Then
            Console.WriteLine( _
                "No manifest resources found in image ""{0}"".", _
                Path.GetFullPath(Manager.FileName) _
            )
            Return
        End If
        Console.WriteLine( _
            "Manifest resources found in image ""{0}"":", _
            Path.GetFullPath(Manager.FileName) _
        )
        Dim Output As New StringBuilder()
        For Each Resource As ResourceInfo In Resources
            With Output
                .Append(Environment.NewLine)
                .Append( _
                    "Resource " & Resource.Id.ToString() & " (" & NativeMethods.LangIdToString(Resource.Language) & "):" & _
                    Environment.NewLine & Environment.NewLine _
                )
                .Append(Encoding.UTF8.GetString(Resource.GetData()))
            End With
        Next Resource
        Console.WriteLine(Output.ToString())
    End Sub

    ''' <summary>
    '''   Writes an error message onto the console.
    ''' </summary>
    ''' <param name="Message">
    '''   The error message to be written onto the console.
    ''' </param>
    Private Sub WriteError(ByVal Message As String)
        Console.WriteLine(Message.TrimEnd(New Char() {ControlChars.Cr, ControlChars.Lf}))
    End Sub

    ''' <summary>
    '''   Writes information about the application onto the console.
    ''' </summary>
    Private Sub WriteLogo()
        Console.WriteLine( _
            "{0} version {1}" & Environment.NewLine & _
            "{2}" & Environment.NewLine, _
            My.Application.Info.ProductName, _
            My.Application.Info.Version, _
            My.Application.Info.Copyright _
        )
    End Sub

    ''' <summary>
    '''   Writes information about the usage of the application onto the
    '''   console.
    ''' </summary>
    Private Sub WriteUsage()
        Console.WriteLine( _
            "Usage: ThemeUtil ExecutableFile [ManifestFile [ResourceID]]" & Environment.NewLine & _
            Environment.NewLine & _
            "  ExecutableFile     Path and filename of the executable file to work with." & Environment.NewLine & _
            "  ManifestFile       Path and filename of manifest file to embed in" & Environment.NewLine & _
            "                     'ExecutableFile'." & Environment.NewLine & _
            "                     If missing, all current manifest resources in" & Environment.NewLine & _
            "                     'ExecutableFile' will be listed." & Environment.NewLine & _
            "  NumericResourceID  The resource identifier that should be used as identifier" & Environment.NewLine & _
            "                     for the manifest resource." & Environment.NewLine & _
            "                     The default is 1 ('CREATEPROCESS_MANIFEST_RESOURCE_ID')." _
        )
    End Sub
End Module